#pragma once

#include <cglib/core/parameters.h>

struct CTwBar;
int get_default_landscape_quality_reduction();
/*
 * Raytracing parameters.
 *
 * Have a look at cglib/parameters.h to see which parameters are built-in already.
 * Feel free to add more parameters here. You can add them to the AntTweakBar gui,
 * aswell, to make interactive tweaking easier.
 */
class DeviceRenderingParameters : public Parameters
{
public:
	enum Renderers {
		SHADOW_MAP,
		FIRE_PLACE,
		PROCEDURAL_LANDSCAPE
	};
	DeviceRenderingParameters() {}
	~DeviceRenderingParameters() {}

	/* restart means reinitialization for device rendering */
	virtual bool derived_change_requires_restart(Parameters const& old_) const override final;

	virtual void derived_gui_setup(CTwBar* main_bar) final;

	float near_clip = 0.05f, far_clip = 200.0f;

	float fov = 60.0f;
	Renderers current_renderer = SHADOW_MAP;

	bool enable_animation = true;
	
	struct Shadow {
		float shadow_bias     = 0.003f;
		float light_near_clip = 10.0f;
		float light_far_clip  = 65.0f;
		float light_fov       = 45.0f;
		glm::vec3 light_direction = glm::vec3(-0.741693f, -0.66015f, 0.11872f);
	} shadow_map;

	struct Landscape {
		bool visualize_heightmap = false;
		bool visualize_reflection = false;

		bool no_water_surface = false;
		bool no_textures = false;
		bool wireframe = false;

		bool night_time = false;
		int quality_reduction = get_default_landscape_quality_reduction();

		struct Water
		{
			float height = 30.0f;
			float wave_scaling = 1.4f;
			float reflection_perturbation = 1.0f;
			glm::vec3 color = glm::vec3(0.6f, 0.8f, 0.8f);
			float absorption = 0.2f;
			float surface_absorption = 1.5f;
			bool enable_fresnel = true;
		} water;

		struct Terrain
		{
			float height_scaling = 100.0f;
			float beach_margin = 2.0f;
			float min_rock_threshold = 0.5f;
			float rock_blend_margin = 0.25f;

			float lacunarity = 2.01f;
			float weight_factor = 0.5f;
			float angle_factor = 121.5f; // degrees
			float valley_smoothness = 0.65f;
		} terrain;
	} landscape;
};
